/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */
#define DEBUG_SHOPPER 1
#include "shopper.h"           // automake, i8n, gettext

#include "CategoryDialog.h"
#include "notify.h"

namespace Shopper
{
////////////////////////////////////////////////////////////////
// Public
	CategoryManage::CategoryManage(QWidget * parent, Shopper::List* l) :
		CategoryDialog(parent, l, NULL)
	{}

////////////////////////////////////////////////////////////////
// Private  class CategoryDialog : public QDialog
	CategoryDialog::CategoryDialog(QWidget * parent, Shopper::List* l, Shopper::Category* cat) :
		QDialog(parent),
		mylist(l),
		mycat(cat),
		area(NULL)
	{
		_ENTER;
		// Buttons
		bbox = new QDialogButtonBox(Qt::Horizontal, this);
		bbox->addButton(tr("Done"), QDialogButtonBox::AcceptRole);
		connect(bbox, SIGNAL(accepted()), this, SLOT(accept()));
		
		connect(bbox->addButton(tr("Add Category"), QDialogButtonBox::ActionRole),
				SIGNAL(clicked()),
				this, SLOT(add_category()));

		// Scroll area with category views
		areaS = new QScrollArea(this);
		areaS->setWidgetResizable(true);

		// moved from update
		area = new QWidget();
		areaL = new QVBoxLayout(area);
		area->setLayout(areaL); // set the layout and take parentage of all children
		areaS->setWidget(area); // add area to the scroller

		// Build them up
		vbox = new QVBoxLayout(this);
		vbox->addWidget(areaS);
		vbox->addWidget(bbox);

		// add the categories to the area.
		update_area();

		// Now connect the List's  category_list_changed() signal to this object's updater
		connect (l, SIGNAL(category_list_changed()),
				 this, SLOT(update_area()));

		// geometric aesthetics
//		int h=get_screen()->get_height();
//		int w=get_screen()->get_width();
//		resize((w*6)/10,h);
	}

	void CategoryDialog::update_area()
	{
		_ENTER;
		// Clean up our old
		QObject *child;
		foreach (child, area->children()){
			if (child->isWidgetType()) {
				DEBUG("Kill a child " << child->metaObject()->className()  );
				child->deleteLater();
			}
		}
		
		// lots of these
		CatView *cv;
		int pos;
		Shopper::List::pCategoryIter begin =mylist->categoriesI(),
			end = mylist->categoriesEnd();
		Shopper::List::pCategoryIter catI;
		for(catI = begin, pos=0; catI != end; ++catI,++pos)
		{
			cv = new CatView(**catI, *mylist);
			areaL->addWidget(cv);   // add CatView to the layout
			connect(cv, SIGNAL(move_up()),
					this, SLOT(move_up()));
			connect(cv, SIGNAL(move_down()),
					this, SLOT(move_down()));
			if (catI == begin) cv->set_pos(pos,-1); // size of -1 won't be reached
		}
		cv->set_pos(pos-1,pos-1); // Now we know the end
		
		area->show();
	}
	
	void CategoryDialog::add_category()
	{
		// Create a new category
		mycat = new Shopper::Category("New Category");
		mylist->add(*mycat);
		// FIXME: set the sw->scroll and category edit focus here
	}
#define DOWN_SCREEN 1
#define UP_SCREEN -1
	void CategoryDialog::move_up()
	{
		_ENTER;
		CatView *src = static_cast<CatView*>(this->sender());
		move(src, UP_SCREEN);
	}
	void CategoryDialog::move_down()
	{
		_ENTER;
		CatView *src = static_cast<CatView*>(this->sender());
		move(src, DOWN_SCREEN);
	}
	void CategoryDialog::move(CatView* src, int direction)
	{
		_ENTER;
		int pos = areaL->indexOf(src);
		int max = areaL->count()-1;
		DEBUG("src at pos=" << pos << "/"<<max<<" direction=" << direction);
		
		if (direction == DOWN_SCREEN and pos == max) return;	// moving 'down' the screen
		if (direction == UP_SCREEN   and pos == 0)  return;
		
		QLayoutItem *li = areaL->itemAt(pos+direction);
		if (li == 0) {
			DEBUG("No item found at pos=" << pos << " direction=" << direction);
			return;
		}
		CatView *that = static_cast<CatView*>(areaL->itemAt(pos+direction)->widget());
		Q_ASSERT(that != 0);

		areaL->removeWidget(src);
		areaL->insertWidget(pos+direction, src);
		mylist->swap_categories(src->get_cat(), that->get_cat());
		src->set_pos(pos+direction,max);
		that->set_pos(pos,max);
		area->show();
	}

////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////

// The widget shows the state of a category
// it can move up or down and edits the relevant category.
	CatView::CatView(Category &cat, List &lst, QWidget *parent) :
		QWidget(parent),
		mycat(&cat),
		mylist(&lst)
	{
		_ENTER;
		// UI
		name = new LabelEntry(cat.get_name(), this);
//		up_b = new QPushButton(style()->standardIcon(QStyle::SP_ArrowUp), "", this);
//		down_b = new QPushButton(style()->standardIcon(QStyle::SP_ArrowDown), "", this);
		up_b = new QToolButton(this);
		up_b->setArrowType(Qt::UpArrow);
		down_b = new QToolButton(this);
		down_b->setArrowType(Qt::DownArrow);
//		QPushButton *del_b = new QPushButton(style()->standardIcon(QStyle::SP_TrashIcon), "", this);
		QToolButton *del_b = new QToolButton(this);
		del_b->setIcon(style()->standardIcon(QStyle::SP_TrashIcon));
		QHBoxLayout *box = new QHBoxLayout(this);

		box->addWidget(del_b);
		box->addWidget(name,1);
		box->addWidget(up_b);
		box->addWidget(down_b);
		setLayout(box);
		
		// Respond to changing the label
		connect(name, SIGNAL(changed()),
				this, SLOT(name_changed()));
		// Respond to del
		connect(del_b, SIGNAL(clicked()),
				this, SLOT(delete_category()));
		// Respond to up
		connect(up_b, SIGNAL(clicked()),
				this, SIGNAL(move_up()));
		// Respond to down
		connect(down_b, SIGNAL(clicked()),
				this, SIGNAL(move_down()));
		
		// Make the container visible according to visibility logic
		DEBUG("CatView created for" <<cat.get_name());
	}

	void CatView::delete_category()
	{
		DEBUG("Delete Category");
		if (mylist->is_category_active(*mycat)) {
			notify->showMessage("Shopper: Info", "Can't delete current category");
			return;
		}
		
		// Message
		QString l;
		int s  = mycat->get_size();
		if (s > 0) {
			l =  "Category '" + mycat->get_name()
				+ "' has " + QString(s)
				+ " items. Delete them all? ";
		} else {
			l = "Delete empty category '" + mycat->get_name() + "'?";
		}
		
		QDialog dia(this);
		QVBoxLayout vbox(&dia);
		QDialogButtonBox bbox(QDialogButtonBox::Yes | QDialogButtonBox::No, Qt::Horizontal, &dia);
		connect(&bbox, SIGNAL(accepted()), &dia, SLOT(accept()));
		connect(&bbox, SIGNAL(rejected()), &dia, SLOT(reject()));
		vbox.addWidget(new QLabel("Are you sure?", &dia));
		vbox.addWidget(new QLabel(l, &dia));
		vbox.addWidget(&bbox);
		dia.setLayout(&vbox);

		if (dia.exec() == QDialog::Accepted)
		{
//FIX			Hildon::Banner::show_information(*this, "Deleted category " + mycat->get_name());
			mylist->rm(*mycat);
		}
	}

	void CatView::name_changed()
	{
		mycat->set_name(name->getText()); 
	}
	void CatView::set_pos(int pos, int size)
	{
		_ENTER;
		if (pos == 0) up_b->hide(); else up_b->show();
		if (pos == size) down_b->hide(); else down_b->show();
		show();
	}
	
	Category * CatView::get_cat()
	{
		return mycat;
	}
}
