/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */
#define DEBUG_SHOPPER 1
#include "shopper.h"           // automake, i8n, gettext
#include "LabelEntry.h"
#include <QtGui>
#include "GestureWatcher.h"

namespace Shopper
{
	// SLabel a label that emits pressed() when clicked
	SLabel::SLabel(QWidget * parent, Qt::WindowFlags f ) :
		QLabel(parent, f)
	{
	}
	SLabel::SLabel( const QString & text, QWidget * parent, Qt::WindowFlags f ) :
		QLabel(text, parent, f)
	{
	}
	void SLabel::setVisible(bool vis)
	{
		_ENTER;
		QLabel::setVisible(vis);
	}
	void SLabel::mouseReleaseEvent ( QMouseEvent * event )
	{
		_ENTER;
		if (event->button() == Qt::LeftButton) {
			event->accept();
			emit pressed();
		}
		event->ignore();
	}

	// LabelEntry - a label that turns into a textbox when clicked
	LabelEntry::LabelEntry(QWidget *parent) :
		QWidget(parent),
		data(""),
		entry(NULL),
		layout(this)
	{
		init();
	}
	LabelEntry::LabelEntry(const QString & value, QWidget *parent) :
		QWidget(parent),
		data(value),
		entry(NULL),
		layout(this)
	{
		init();
	}
	void LabelEntry::init()
	{
		label = new SLabel(data, this);
		GestureWatcher* gw = GestureWatcher::getGestureWatcher();
		gw->connect(label, Gesture("r l r "),
				   this, SLOT(label_clicked()));
//		connect(label, SIGNAL(pressed()),
//				this, SLOT(label_clicked()));
		layout.addWidget(label);
		layout.setContentsMargins(0, 0, 0, 0);
		label->show();
	}
	QString LabelEntry::getText()
	{
		return data;
	}

	void LabelEntry::label_clicked()
	{
		_ENTER;
		if (entry != NULL) return; // events may arrive post-creation
		entry = new QLineEdit(this);
		entry->setSizePolicy(label->sizePolicy());
//		entry->set_activates_default(false);
//		entry->set_width_chars(15); // FIXME
		entry->setText(data);
		// self-destruct on unfocus or activate
		connect(entry, SIGNAL(editingFinished()),
				this, SLOT(entry_finished()));
	
		layout.removeWidget(label);
		label->hide();
		layout.addWidget(entry);
		entry->show();
		entry->setFocus(Qt::MouseFocusReason);
	}
	void LabelEntry::entry_finished()
	{
		_ENTER;
		if (entry == NULL) return; // events may arrive post-deletion
		data=entry->text();
		label->setText(data);
		layout.removeWidget(entry);
		entry->hide();
		layout.addWidget(label);
		label->show();
		entry->deleteLater();   // We can now forget about it...
		entry=NULL;
		emit changed();
	}
	void LabelEntry::setAlignment ( Qt::Alignment al )
	{
		label->setAlignment(al);
	}
	void LabelEntry::setText ( const QString & d )
	{
		data = d;
	}
	void LabelEntry::setVisible(bool vis)
	{
		_ENTER;
		QWidget::setVisible(vis);
	}

	void LabelEntry::setFrameStyle(int i)
	{
		label->setFrameStyle(i);
	}
	void LabelEntry::setSizePolicy(QSizePolicy::Policy h, QSizePolicy::Policy v)
	{
		QWidget::setSizePolicy(h,v);
		label->setSizePolicy(h,v);
	}
	void LabelEntry::setSizePolicy(QSizePolicy p)
	{
		_ENTER;
		QWidget::setSizePolicy(p);
		label->setSizePolicy(p);
	}
}
