/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
//#define DEBUG_SHOPPER 1
#include "shopper.h"           // automake, i8n, gettext
#include "shopperList.h"
#include "ItemDialog.h"
#include "LabelEntry.h"
#include "shopperView.h"
#include <QtGui>
#include <iostream>
#include "GestureWatcher.h"

using namespace std;

namespace Shopper
{
// The widget shows the state of an item.
	// class CategoryView: public QWidget
	CategoryView::CategoryView(Category &cat, List &l, QWidget *parent) :
		QWidget(parent),
		mycat(&cat),
		mylist(&l),
		outer_b()
	{
		QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
		sizePolicy.setHorizontalStretch(1);
		sizePolicy.setVerticalStretch(0);
		setSizePolicy(sizePolicy);
//		setFrameStyle(QFrame::Panel|QFrame::Sunken);
		name = new SLabel(mycat->get_name(), this);
		name->setAlignment(Qt::AlignCenter);
		name->setSizePolicy(sizePolicy);
		// Lay them out
		outer_b.addWidget(name);
		outer_b.setContentsMargins(0, 0, 0, 0);
		// and apply
		setLayout(&outer_b);

		// Now connect the item's changed signal to this object's updater
		connect(&cat, SIGNAL(changed()),
				this, SLOT(category_changed()));
		show();
		// Make the container visible according to visibility logic
		DEBUG("ItemView created for " <<it.get_name());
	}

	void CategoryView::category_changed()
	{
		name->setText(mycat->get_name());
		show();
	}
	
	void CategoryView::setVisible(bool vis)
	{
		// Hide if not wanted
		if (((mylist->get_state() == WHATS_LEFT) || (mylist->get_state() == OUT_SHOPPING))
			&& mycat->wanted() == 0) {
			DEBUG(" shopping, none wanted, hide\n");
			QWidget::setVisible(false);
			return;
		}		

		if ((mylist->get_state() == WHATS_LEFT) && mycat->bought()==mycat->wanted()) {
			DEBUG(" what's left + all bought, hide\n");
			QWidget::setVisible(false);
			return;
		}
		DEBUG(" show\n");
		QWidget::setVisible(vis);
		return;
	}

////////////////////////////////////////////////////////////////
	ItemView::ItemView(Item &it, List &l, QWidget *parent) :
		QWidget(parent),
		myitem(&it),
		mylist(&l),
		outer_b(),
		inner_b()
	{
		QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
		sizePolicy.setHorizontalStretch(1);
		sizePolicy.setVerticalStretch(0);
		setSizePolicy(sizePolicy);
//		setFrameStyle(QFrame::Panel|QFrame::Sunken);
		tick = new QCheckBox(this);
		// The tick represents either wanted or bought
		tick->setChecked((l.get_state() == MAKING_LIST) ?
						it.get_wanted() : it.get_bought());
		desc = new SLabel(it.get_desc(), this);
		desc->setAlignment(Qt::AlignLeft);
		desc->setSizePolicy(sizePolicy);
//		desc->setFrameStyle(QFrame::Panel|QFrame::Sunken);
		note = new LabelEntry(it.get_note(),this);
		note->setAlignment(Qt::AlignRight);
		note->setSizePolicy(sizePolicy);
//		note->setFrameStyle(QFrame::Panel|QFrame::Sunken);
		// Lay them out
		outer_b.addWidget(tick);
		outer_b.addLayout(&inner_b,1);
		inner_b.addWidget(desc, 1);
		inner_b.addWidget(note, 1);

		outer_b.setContentsMargins(0, 0, 0, 0);
		inner_b.setContentsMargins(0, 0, 0, 0);
		// and apply
		setLayout(&outer_b);

		// Respond to user clicking the checkbox.
		connect(tick, SIGNAL(stateChanged(int)),
				this, SLOT(tick_changed(int)));
		// And clicking the description is the same.
		GestureWatcher* gw = GestureWatcher::getGestureWatcher();
		gw->connect(desc, Gesture("r l "),
				   this, SLOT(desc_clicked()));
		gw->connect(desc, Gesture("l r "),
				   this, SLOT(desc_clicked()));
//		connect(desc, SIGNAL(pressed()),
//				this, SLOT(desc_clicked()));
		// Respond to clicking the note
		connect(note, SIGNAL(changed()),
				this, SLOT(note_changed()));
		
		// Now connect the item's changed signal to this object's updater
		connect(&it, SIGNAL(changed()),
				this, SLOT(item_changed()));
		connect(&it, SIGNAL(deleted()),
				this, SLOT(item_deleted()));

		show();
//		DEBUG("ItemView created for " <<it.get_desc());
	}
	
	// Prepare menu actions at time of popup
	void ItemView::contextMenuEvent( QContextMenuEvent * event )
	{
		QMenu menu(this);
		connect(menu.addAction(tr("Edit")),
				SIGNAL(triggered()), this, SLOT(edit_item()));
		connect(menu.addAction(tr("Delete")),
				SIGNAL(triggered()), this, SLOT(delete_item()),
				Qt::QueuedConnection);
		menu.exec(event->globalPos());
	}

// Only shows if matching item should be seen.
	void ItemView::setVisible(bool vis)
	{
		// Hide if not wanted
		if (((mylist->get_state() == WHATS_LEFT) || (mylist->get_state() == OUT_SHOPPING))
			&& ! myitem->get_wanted()) {
			DEBUG(" shopping, not wanted, hide\n");
			QWidget::setVisible(false);
			return;
		}		

		if ((mylist->get_state() == WHATS_LEFT) && myitem->get_bought()) {
			DEBUG(" what's left+bought, hide\n");
			QWidget::setVisible(false);
			return;
		}
		DEBUG(" show\n");
		QWidget::setVisible(vis);
		return;
	}

	void ItemView::note_changed()
	{
		myitem->set_note(note->getText()); // Setting the data
		                                   // will call an
		                                   // update on the note
	}
	void ItemView::edit_item()
	{
		ItemEdit itemD(this, mylist, myitem);
		itemD.exec();
	}
	void ItemView::delete_item()
	{
		DEBUG("Delete Item\n");
		mylist->rm(*(this->myitem));
	}
	void ItemView::desc_clicked()
	{
		// Make the tick tock
		DEBUG("Desc pressed\n");
		tick->toggle(); // Invokes tick_changed
		tick->setFocus(Qt::MouseFocusReason);
	}
	void ItemView::tick_changed(int s)
	{
		bool t = (s == Qt::Checked);
		// Suspend our data watch - we're making the change and don't need
		// to be notified
		DEBUG("Tick " << t <<"\n");
		switch (mylist->get_state()) {
		case OUT_SHOPPING:
			DEBUG("shopping ");
		case WHATS_LEFT:
			DEBUG("what's left\n");
			myitem->set_bought(t);
			break;
		case MAKING_LIST:
			DEBUG("Making list\n");
			myitem->set_wanted(t);
			break;
		default:
			DEBUG("State invalid\n");
		}
	}

// When the underlying data changes, 
	void ItemView::item_changed()
	{
		DEBUG("Notified of item change: updating view\n");
		tick->setChecked((mylist->get_state() == MAKING_LIST) ?
						myitem->get_wanted() : myitem->get_bought());
		desc->setText(myitem->get_desc());
		note->setText(myitem->get_note());
		if (! mylist->is_category_active(*(myitem->get_category())))
			hide();
		else
			show();
	}
	
	void ItemView::item_deleted()
	{
		DEBUG("Item deleted, thanks for letting me know but this is BAD");
	}

// This class displays and manages a list.
//
// It creates ItemViews dynamically
// When the mode changes it updates the display
// When the active category changes, it updates the display
// 
// class ListView: public QScrollArea
	
	ListView::ListView(List &l, QWidget *parent) :
		QFingerScrollArea(parent),
		mylist(&l),
		contents(NULL)
	{
//		setBackgroundRole(QPalette::Light);
		setWidgetResizable(true);
		QSizePolicy sizePolicy(QSizePolicy::Ignored, QSizePolicy::Preferred);
		sizePolicy.setHorizontalStretch(1);
		setSizePolicy(sizePolicy);

		update();

		
		// Now connect the List's state changed signal to this object's updater
		connect(&l, SIGNAL(state_changed()),
				this, SLOT(list_changed()));
		connect(&l, SIGNAL(category_list_changed()),
				this, SLOT(list_changed()));
		connect(&l, SIGNAL(item_list_changed()),
				this, SLOT(list_changed()));
		connect(&l, SIGNAL(active_category_changed()),
				this, SLOT(list_changed()));
		connect(&l, SIGNAL(changed()),
				this, SLOT(list_changed()));
	}

	ListView::~ListView()
	{
		std::map<int, ItemView*>::iterator  itemVsI;
		
		for(itemVsI = shownItemVs.begin();
			itemVsI != shownItemVs.end(); ++itemVsI)
		{
			itemVsI->second->deleteLater();
		}
	}

	void ListView::filter_by(Category &c)  // Only view certain categories
	{
		mylist->make_category_active(c);
	}

	void ListView::list_changed()
	{
		_ENTER;
		DEBUG("Notified of a list_change");
		update();
	}

	void ListView::update()
	{
		if (contents!=NULL) contents->deleteLater();

		contents = new QFrame;
		vbox = new QVBoxLayout;
		contents->setLayout(vbox);
//		contents->setFrameStyle(QFrame::Panel|QFrame::Raised);
		contents->setLineWidth(3);

		// Scans all the items in the Shopper::List and creates or
		// removes them from the items map
		ItemView *w;
		CategoryView *cw;
		DEBUG("Updating: active category is \n");
		for(Shopper::List::pCategoryIter catI = mylist->categoriesI();
			catI != mylist->categoriesEnd(); ++catI)
		{
			DEBUG("   " << (*catI)->get_name() << " " << ((mylist->is_category_active(**catI))?"  *":""));
		}
		// Delete all the existing itemviews and then clear the map
		// FIXME: do we even need a map now?
//		for(std::map<const int, ItemView*>::iterator it = shownItemVs.begin() ; it!= shownItemVs.end(); it++)
//		{
//			it->second->deleteLater();
//		}
//		shownItemVs.clear();

		for(Shopper::List::pCategoryIter cI = mylist->categoriesI();
			cI != mylist->categoriesEnd(); ++cI)
		{
			if (mylist->is_category_active(*(*cI)))
			{
				Shopper::Category *c = *cI;
				DEBUG(" ... active");
				if (true) {
//				if (showCategoryHeadings) {
					cw = new CategoryView(*c, *mylist, contents);
					vbox->addWidget(cw, 1);
				}
				for(Shopper::Category::pItemIter itemI = c->itemsI();
					itemI != c->itemsEnd(); ++itemI)	{
					w = new ItemView(**itemI, *mylist, contents);
					// Keep track of our child widgets
//				shownItemVs.insert(pair<int,ItemView*>((*itemI)->get_id(), w));
					// Pack from the start...
					vbox->addWidget(w, 1);
					DEBUG(" ... made new itemView");
				}
			} else {
				DEBUG(" ... not selected");
			}
		}
		vbox->addStretch(1);
		QSizePolicy sizePolicy(QSizePolicy::Ignored, QSizePolicy::Preferred);
		sizePolicy.setHorizontalStretch(1);
		contents->setSizePolicy(sizePolicy);
		DEBUG("\n\nAbout to setWidget()");
		setWidget(contents);
		DEBUG("finished setWidget()\n\n");
	}
}
