/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef __SHOPPERVIEW_H__
#define __SHOPPERVIEW_H__

#include "shopperList.h"
#include "LabelEntry.h"
#include <map>

#include "QFingerScrollArea.h"
#include <QScrollArea>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QCheckBox>

using namespace std;
namespace Shopper
{
	class CategoryView: public QWidget
	{
		Q_OBJECT;
		
	public:
		CategoryView(Shopper::Category &it, Shopper::List &l, QWidget *parent);
	
		virtual void setVisible(bool vis);
		void update();
		QHBoxLayout      outer_b;
		SLabel           *name;       /* This displays the category description */
		Category         *mycat;      /* the category being displayed */
		List             *mylist;     /* the parent list */
		

	public slots:
		// data monitors
		void category_changed();
	};

	class ItemView: public QWidget
	{
		Q_OBJECT;
		
	public:
		ItemView(Shopper::Item &it, Shopper::List &l, QWidget *parent);
	
		virtual void setVisible(bool vis);
		void update();
		QCheckBox *tick;       /* display either the 'bought/not
									  * bought' or the 'wanted/not wanted'
									  * state depending on the current
									  * mode.  This also includes the
									  * label
									  */
		QHBoxLayout      outer_b;
		QHBoxLayout      inner_b;
		SLabel           *desc;       /* This displays the item description */
		LabelEntry       *note;       /* This displays the notes. When
									   * clicked on it transforms into a
									   * textEntry */
		Item            *myitem;     /* the item being displayed */
		List            *mylist;     /* the parent list */
		
		// This handles right clicks (or equiv)
		void contextMenuEvent( QContextMenuEvent * event );

	public slots:
		// signal handlers
		void desc_clicked();     // toggles the check
		void tick_changed(int s);// changes the state when check changes
		void note_changed();     // EntryLabel calls to change note 
		void edit_item();        // menu calls this to open edit dialog
		void delete_item();      // menu calls this to removes item
	
		// data monitors
		void item_changed();
		void item_deleted();
	};

// This class displays and manages a list.

// possibly use scrolledwindow or an viewport for finger scrolling
//	class ListView: public QVBoxLayout
	class ListView: public QFingerScrollArea
	{
		Q_OBJECT;
	public:
		ListView(Shopper::List &l, QWidget *parent);
		~ListView();

		void filter_by(Shopper::Category &c);  // Only view certain categories
		void update(); // should be private but show_all() is buggy AFAICT
	private:
		Shopper::List        *mylist;
		std::map<const int, ItemView*>  shownItemVs;       // Items currently being
		                                  // displayed (may be hidden
		                                  // in what's left mode)
//		sigc::connection watch_item;    
//		sigc::connection watch_list;

		QFrame     *contents;     // This widget has a layout and contains all the items
		QVBoxLayout *vbox;     // and is scrolled by this widget
		

	public slots:
		// signal handlers
		void list_changed();
	};
}

#endif /* __SHOPPERVIEW_H__ */
